<?php
/**
 * Booking Order management class
 *
 * manage order processes for Booking products
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Order' ) ) {
    /**
     * @class YITH_WCBK_Order
     *
     * @since 1.0.0
     */
    class YITH_WCBK_Order {

        /**
         * Single instance of the class
         *
         * @var \YITH_WCBK_Order
         * @since 1.0.0
         */
        protected static $_instance;

        public static $order_item_data_prefix = 'yith_booking_';

        public static $order_bookings_meta = 'yith_bookings';

        /**
         * Returns single instance of the class
         *
         * @return \YITH_WCBK_Order
         * @since 1.0.0
         */
        public static function get_instance() {
            if ( is_null( self::$_instance ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }

        public function __construct() {
            // Add booking data in order item meta
            $woocommerce_new_order_item_hook = version_compare( WC()->version, '3.0.0', '>=' ) ? 'woocommerce_new_order_item' : 'woocommerce_add_order_item_meta';
            add_action( $woocommerce_new_order_item_hook, array( $this, 'woocommerce_add_order_item_meta' ), 10, 2 );

            add_action( 'woocommerce_checkout_order_processed', array( $this, 'check_order_for_booking' ), 999, 2 );
            add_action( 'yith_wcbk_check_order_with_booking', array( $this, 'check_order_for_booking' ), 999, 2 );

            add_action( 'woocommerce_order_status_completed', array( $this, 'set_booking_as_paid' ) );
            add_action( 'woocommerce_order_status_processing', array( $this, 'set_booking_as_paid' ) );

            add_action( 'woocommerce_order_details_after_order_table', array( $this, 'show_related_bookings' ) );

            /** cancel booking if the related order is cancelled
             *
             * @since 1.3.1
             */
            add_action( 'woocommerce_order_status_cancelled', array( $this, 'set_booking_as_cancelled' ) );
        }


        /**
         * show related bookings in order table
         *
         * @param WC_Order $order
         */
        public function show_related_bookings( $order ) {
            $order_id = yit_get_prop( $order, 'id', true, 'edit' );
            $bookings = YITH_WCBK()->booking_helper->get_bookings_by_order( $order_id );
            $bookings = apply_filters( 'yith_wcbk_order_bookings_related_to_order', $bookings, $order );
            if ( !!$bookings ) {
                echo '<h2>' . __( 'Related Bookings', 'yith-booking-for-woocommerce' ) . '</h2>';
            }
            do_action( 'yith_wcbk_show_bookings_table', $bookings );
        }

        /**
         * add meta in order
         *
         * @access public
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com>
         *
         * @param $item_id
         * @param $values
         */
        public function woocommerce_add_order_item_meta( $item_id, $values ) {
            $booking_data = false;

            if ( isset( $values[ 'yith_booking_data' ] ) && is_array( $values[ 'yith_booking_data' ] ) ) {
                $booking_data = $values[ 'yith_booking_data' ];
            } elseif ( isset( $values->legacy_values[ 'yith_booking_data' ] ) && is_array( $values->legacy_values[ 'yith_booking_data' ] ) ) {
                $booking_data = $values->legacy_values[ 'yith_booking_data' ];
            }

            if ( $booking_data ) {
                /* Parse person types to get post titles to prevent error if a person type is deleted */
                if ( !empty( $booking_data[ 'person_types' ] ) ) {
                    $booking_data_person_types = array();
                    foreach ( $booking_data[ 'person_types' ] as $person_type_id => $person_type_number ) {
                        $person_type_title           = get_the_title( $person_type_id );
                        $booking_data_person_types[] = array(
                            'id'     => $person_type_id,
                            'title'  => $person_type_title,
                            'number' => $person_type_number,
                        );
                    }
                    $booking_data[ 'person_types' ] = $booking_data_person_types;
                }

                /* Parse booking services to get post titles to prevent error if a person type is deleted */
                if ( !empty( $booking_data[ 'booking_services' ] ) ) {
                    $booking_data_services = array();
                    foreach ( $booking_data[ 'booking_services' ] as $service_id ) {
                        $service = yith_get_booking_service( $service_id );
                        if ( $service->is_valid() ) {
                            $booking_data_services[]      = array(
                                'id'     => $service_id,
                                'title'  => $service->get_name(),
                                'hidden' => $service->is_hidden(),
                            );
                            $booking_data[ 'services' ][] = $service_id;
                        }
                    }
                    $booking_data[ 'booking_services' ] = $booking_data_services;
                }

                /* add booking data: will be hidden because it's array */
                wc_add_order_item_meta( $item_id, 'yith_booking_data', $booking_data );

                // add booking id data if booking required confirmation and it's confirmed
                if ( isset( $booking_data[ '_booking_id' ] ) ) {
                    wc_add_order_item_meta( $item_id, '_booking_id', $booking_data[ '_booking_id' ] );

                }

                $add_details = apply_filters( 'yith_wcbk_order_add_booking_details_in_order_item',
                                              isset( $values[ 'yith_wcbk_show_details' ] ) && $values[ 'yith_wcbk_show_details' ],
                                              $item_id, $values );
                if ( $add_details ) {
                    /* Add booking data to display in order */
                    foreach ( $booking_data as $booking_data_key => $booking_data_value ) {
                        $this_title = yith_wcbk_get_booking_meta_label( $booking_data_key );

                        switch ( $booking_data_key ) {
                            case 'person_types':
                                if ( is_array( $booking_data_value ) ) {
                                    foreach ( $booking_data_value as $person_type ) {
                                        if ( !empty( $person_type[ 'number' ] ) )
                                            wc_add_order_item_meta( $item_id, $person_type[ 'title' ], $person_type[ 'number' ] );
                                    }
                                }
                                break;
                            case 'booking_services':
                                if ( is_array( $booking_data_value ) ) {
                                    $booking_services        = array();
                                    $hidden_booking_services = array();
                                    foreach ( $booking_data_value as $service ) {
                                        if ( !$service[ 'hidden' ] ) {
                                            $booking_services[] = $service[ 'title' ];
                                        } else {
                                            $hidden_booking_services[] = $service[ 'title' ];
                                        }
                                    }
                                    if ( !!$booking_services ) {
                                        wc_add_order_item_meta( $item_id, __( 'Booking Services', 'yith-booking-for-woocommerce' ), implode( ', ', $booking_services ) );
                                    }
                                    if ( !!$hidden_booking_services ) {
                                        wc_add_order_item_meta( $item_id, '_hidden_booking_services', implode( ', ', $hidden_booking_services ) );
                                    }
                                }
                                break;

                            case 'from':
                            case 'to':
                                $this_value = date_i18n( wc_date_format(), $booking_data_value );
                                wc_add_order_item_meta( $item_id, $this_title, $this_value );
                                break;
                            case 'duration':
                                $this_value = $booking_data_value;
                                $product_id = isset( $values[ 'product_id' ] ) ? $values[ 'product_id' ] : 0;
                                $product    = wc_get_product( $product_id );
                                if ( $product && $product instanceof WC_Product_Booking ) {
                                    $duration_unit       = $product->get_duration_unit();
                                    $duration_unit_label = yith_wcbk_get_duration_unit_label( $duration_unit, absint( $booking_data_value ) );
                                    $this_value          .= ' ' . $duration_unit_label;
                                }
                                wc_add_order_item_meta( $item_id, $this_title, $this_value );
                                break;
                            default:
                                wc_add_order_item_meta( $item_id, $this_title, $booking_data_value );
                                break;
                        }
                    }
                }

            }
        }

        /**
         * Check if order contains booking products.
         * If it contains a booking product, it will create the booking
         *
         * @param int   $order_id
         * @param array $posted
         */
        public function check_order_for_booking( $order_id, $posted = array() ) {
            if ( !apply_filters( 'yith_wcbk_order_check_order_for_booking', true, $order_id, $posted ) )
                return;

            $order       = wc_get_order( $order_id );
            $order_items = $order->get_items();
            //check if the bookings are created
            $bookings = get_post_meta( $order_id, self::$order_bookings_meta, true );


            if ( !$order_items ) {
                return;
            }

            foreach ( $order_items as $order_item_id => $order_item ) {
                /**
                 * @var WC_Product_Booking $product
                 */
                $product    = is_callable( array( $order_item, 'get_product' ) ) ? $order_item->get_product() : $order->get_product_from_item( $order_item );
                $product_id = $product ? $product->get_id() : false;
                if ( !$product || !$product_id || !YITH_WCBK_Product_Helper::is_booking( $product_id ) ) {
                    continue;
                }

                $args                  = array();
                $booking_data          = wc_get_order_item_meta( $order_item_id, 'yith_booking_data', true );
                $order_item_booking_id = wc_get_order_item_meta( $order_item_id, '_booking_id', true );
                if ( !$order_item_booking_id && !!$booking_data ) {
                    foreach ( $booking_data as $booking_data_key => $booking_data_value ) {
                        $unserialized_value        = maybe_unserialize( $booking_data_value );
                        $args[ $booking_data_key ] = $unserialized_value;
                    }
                    $product_id              = apply_filters( 'yith_wcbk_booking_product_id_to_translate', $product_id );
                    $args[ 'product_id' ]    = $product_id;
                    $args[ 'title' ]         = $product->get_title();
                    $args[ 'duration_unit' ] = $product->get_duration_unit();
                    $args[ 'order_id' ]      = $order_id;
                    $args[ 'order_item_id' ] = $order_item_id;
                    $args[ 'user_id' ]       = $order->get_user_id();

                    /* ===== C R E A T E   B O O K I N G ===== */
                    $booking = new YITH_WCBK_Booking( '', $args );

                    $bookings = get_post_meta( $order_id, self::$order_bookings_meta, true );
                    $bookings = !!$bookings && is_array( $bookings ) ? $bookings : array();

                    if ( $booking->id ) {
                        wc_add_order_item_meta( $order_item_id, '_booking_id', $booking->id, true );
                        $bookings[] = $booking->id;
                        update_post_meta( $order_id, self::$order_bookings_meta, $bookings );

                        $order->add_order_note( sprintf( __( 'A new booking <a href="%s">#%s</a> has been created from this order', 'yith-booking-for-woocommerce' ), admin_url( 'post.php?post=' . $booking->id . '&action=edit' ), $booking->id ) );

                        do_action( 'yith_wcbk_order_booking_created', $booking, $order, $order_item_id );
                    }
                } elseif ( $order_item_booking_id ) {
                    $booking = yith_get_booking( $order_item_booking_id );
                    if ( $booking && $booking->is_valid() && $booking->has_status( 'confirmed' ) ) {
                        $booking->set( 'order_id', $order_id );
                        $booking_note = sprintf( __( 'Booking associated to order <a href="%s">#%s</a>', 'yith-booking-for-woocommerce' ), admin_url( 'post.php?post=' . $order_id . '&action=edit' ), $order->get_order_number() );
                        $booking->add_note( 'new-order', $booking_note );
                        $booking->update_status( 'unpaid' );
                    }
                }

            }
        }

        /**
         * Set Booking as paid
         *
         * @param $order_id
         */
        public function set_booking_as_paid( $order_id ) {
            $bookings = YITH_WCBK_Booking_Helper()->get_bookings_by_order( $order_id );
            if ( !!( $bookings ) ) {
                foreach ( $bookings as $booking ) {
                    if ( $booking instanceof YITH_WCBK_Booking ) {
                        $booking->update_status( 'paid' );
                    }
                }
            }
        }

        /**
         * Set Booking as cancelled
         *
         * @param $order_id
         */
        public function set_booking_as_cancelled( $order_id ) {
            $bookings = YITH_WCBK_Booking_Helper()->get_bookings_by_order( $order_id );
            if ( !!$bookings ) {
                $order        = wc_get_order( $order_id );
                $order_number = $order ? $order->get_order_number() : $order_id;
                foreach ( $bookings as $booking ) {
                    if ( $booking instanceof YITH_WCBK_Booking ) {
                        $additional_note = sprintf( __( 'Reason: order <a href="%s">#%s</a> has been cancelled.', 'yith-booking-for-woocommerce' ),
                                                    admin_url( 'post.php?post=' . $order_id . '&action=edit' ),
                                                    $order_number );
                        $booking->update_status( 'cancelled', 'change_status', $additional_note );
                    }
                }
            }
        }
    }
}