<?php
/**
 * Product Form Helper class
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Product_Form_Helper' ) ) {
    /**
     * YITH Booking for WooCommerce Premium Product Form Helper
     *
     * @since 1.0.0
     */
    class YITH_WCBK_Product_Form_Helper {
        /** @var YITH_WCBK_Product_Form_Helper */
        protected static $_instance;

        /** @var int */
        private $_current_error_reporting = 0;

        /** @return YITH_WCBK_Product_Form_Helper */
        public static function get_instance() {
            return !is_null( self::$_instance ) ? self::$_instance : self::$_instance = new self();
        }

        public function __construct() {
            add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

            $ajax_actions = array(
                'check_booking_availability',
                'get_booking_price',
                'get_booking_data'
            );

            foreach ( $ajax_actions as $ajax_action ) {
                add_action( 'wp_ajax_yith_wcbk_' . $ajax_action, array( $this, $ajax_action ) );
                add_action( 'wp_ajax_nopriv_yith_wcbk_' . $ajax_action, array( $this, $ajax_action ) );
            }
        }

        /**
         * Enable/disable WP error reporting
         * usefull in AJAX calls with json to prevent JSON parse issue
         *
         * @param bool $enabled
         */
        public function set_error_reporting( $enabled ) {
            if ( $enabled ) {
                error_reporting( $this->_current_error_reporting );
            } else {
                $this->_current_error_reporting = error_reporting();
                error_reporting( 0 );
            }
        }

        /**
         * Echo the string, restore error reporting and die
         * Usefull in AJAC calls
         *
         * @param string $value
         */
        public function echo_and_die( $value ) {
            echo $value;
            $this->_ajax_end();
            die();
        }

        private function _ajax_start() {
            $this->set_error_reporting( false );
            !defined( 'YITH_WCBK_DOING_AJAX' ) && define( 'YITH_WCBK_DOING_AJAX', true );
            !defined( 'YITH_WCBK_DOING_AJAX_FRONTEND' ) && define( 'YITH_WCBK_DOING_AJAX_FRONTEND', true );
        }

        private function _ajax_end() {
            $this->set_error_reporting( true );
        }

        /**
         * Get booking data as Availability and price [AJAX]
         */
        public function get_booking_data() {
            $this->_ajax_start();

            $request = apply_filters( 'yith_wcbk_ajax_booking_data_request', $_POST );

            if ( empty( $request[ 'product_id' ] ) || empty( $request[ 'from' ] ) || ( empty( $request[ 'duration' ] ) && empty( $request[ 'to' ] ) ) ) {
                $error = json_encode( array( 'error' => _x( 'Required POST variable not set', 'Error', 'yith-booking-for-woocommerce' ) ) );
                $this->echo_and_die( $error );
            }

            $date_helper = YITH_WCBK_Date_Helper();

            $product_id = $_POST[ 'product_id' ];
            /**
             * @var WC_Product_Booking $product
             */
            $product = wc_get_product( $product_id );


            if ( $product ) {
                $from = strtotime( $request[ 'from' ] );
                if ( isset( $request[ 'to' ] ) ) {
                    $to = strtotime( $request[ 'to' ] );
                } else {
                    $duration = absint( $request[ 'duration' ] ) * $product->get_booking_prop( 'duration' );
                    $to       = $date_helper->get_time_sum( $from, $duration, $product->get_duration_unit() );
                }

                $is_available = $product->is_available( array( 'from' => $from, 'to' => $to ) );

                $bookable_args = array(
                    'product'  => $product,
                    'bookable' => $is_available,
                    'from'     => $from,
                    'to'       => $to
                );
                ob_start();
                wc_get_template( 'single-product/add-to-cart/bookable.php', $bookable_args, '', YITH_WCBK_TEMPLATE_PATH );
                $message = ob_get_clean();

                $booking_data = array(
                    'is_available' => $is_available,
                    'price'        => $product->get_calculated_price_html( $request ),
                    'message'      => $message,
                );

                $booking_data = apply_filters( 'yith_wcbk_product_form_get_booking_data', $booking_data, $product );

                $booking_data_json = json_encode( $booking_data );
                // Enable display_errors
                $this->echo_and_die( $booking_data_json );
            }

            $error = json_encode( array( 'error' => _x( 'Product not found', 'Error', 'yith-booking-for-woocommerce' ) ) );
            $this->echo_and_die( $error );
        }

        /**
         * Check the availability of booking [AJAX]
         *
         * @param string $type
         *
         * @return bool|void
         */
        public function check_booking_availability( $type = 'AJAX' ) {
            if ( 'AJAX' === $type ) {
                $this->_ajax_start();
            }
            $date_helper             = YITH_WCBK_Date_Helper();
            $required_post_variables = array(
                'product_id',
                'from'
            );
            foreach ( $required_post_variables as $variable ) {
                if ( empty( $_POST[ $variable ] ) ) {
                    if ( $type == 'AJAX' ) {
                        die();
                    } else {
                        return false;
                    }
                }
            }

            if ( empty( $_POST[ 'duration' ] ) && empty( $_POST[ 'to' ] ) ) {
                if ( $type == 'AJAX' ) {
                    die();
                } else {
                    return false;
                }
            }

            $product_id = $_POST[ 'product_id' ];
            /**
             * @var WC_Product_Booking $product
             */
            $product = wc_get_product( $product_id );
            if ( $product && $product->is_type( 'booking' ) ) {

                $from = strtotime( $_POST[ 'from' ] );
                if ( isset( $_POST[ 'to' ] ) ) {
                    $to = strtotime( $_POST[ 'to' ] );
                } else {
                    $duration = absint( $_POST[ 'duration' ] ) * $product->get_booking_prop( 'duration' );
                    $to       = $date_helper->get_time_sum( $from, $duration, $product->get_duration_unit() );
                }

                if ( $type == 'AJAX' ) {
                    echo $product->is_available( array( 'from' => $from, 'to' => $to ) ) ? 'bookable' : 'not_bookable';
                } else {
                    return $product->is_available( array( 'from' => $from, 'to' => $to ) );
                }
            }

            if ( $type == 'AJAX' ) {
                $this->_ajax_end();
                die();
            } else {
                return false;
            }
        }

        /**
         * Get the price of booking [AJAX]
         *
         * @param string $type
         *
         * @return string|void
         */
        public function get_booking_price( $type = 'AJAX' ) {
            if ( 'AJAX' === $type ) {
                $this->_ajax_start();
            }
            $required_post_variables = array(
                'product_id',
            );
            foreach ( $required_post_variables as $variable ) {
                if ( empty( $_POST[ $variable ] ) ) {
                    if ( $type == 'AJAX' ) {
                        die();
                    } else {
                        return '';
                    }
                }
            }

            if ( empty( $_POST[ 'duration' ] ) && empty( $_POST[ 'to' ] ) ) {
                if ( $type == 'AJAX' ) {
                    die();
                } else {
                    return '';
                }
            }

            $product_id = $_POST[ 'product_id' ];
            /**
             * @var WC_Product_Booking $product
             */
            $product = wc_get_product( $product_id );
            if ( $product && $product->is_type( 'booking' ) ) {
                $args = $_POST;

                $price_html = $product->get_calculated_price_html( $args );
                if ( $type == 'AJAX' ) {
                    echo $price_html;
                } else {
                    return $price_html;
                }

            }

            if ( $type == 'AJAX' ) {
                $this->_ajax_end();
                die();
            } else {
                return '';
            }
        }

        /**
         * Frontend enqueue scripts and styles
         */
        public function enqueue_scripts() {
            global $wp_scripts;
            $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

            $jquery_version = isset( $wp_scripts->registered[ 'jquery-ui-core' ]->ver ) ? $wp_scripts->registered[ 'jquery-ui-core' ]->ver : '1.9.2';
            wp_enqueue_style( 'jquery-ui-style', '//ajax.googleapis.com/ajax/libs/jqueryui/' . $jquery_version . '/themes/smoothness/jquery-ui.css' );

            wp_register_script( 'yith-wcbk-booking-form', YITH_WCBK_ASSETS_URL . '/js/booking_form' . $suffix . '.js', array( 'jquery', 'jquery-ui-datepicker', 'jquery-tiptip' ), YITH_WCBK_VERSION, true );
            wp_localize_script( 'yith-wcbk-booking-form', 'yith_booking_form_params', array(
                'ajaxurl'             => admin_url( 'admin-ajax.php' ),
                'i18n_empty_duration' => __( 'Select a duration', 'yith-booking-for-woocommerce' ),
                'i18n_empty_date'     => __( 'Select a date', 'yith-booking-for-woocommerce' ),
                'i18n_min_persons'    => __( 'Minimum people: %s', 'yith-booking-for-woocommerce' ),
                'i18n_max_persons'    => __( 'Maximum people: %s', 'yith-booking-for-woocommerce' ),
            ) );

            wp_register_style( 'yith-wcbk-booking-form-style', YITH_WCBK_ASSETS_URL . '/css/booking-form.css', YITH_WCBK_VERSION );

            $enqueue_form_scripts = false;

            if ( is_admin() ) {
                $screen = get_current_screen();
                if ( 'yith_booking_page_create_booking' === $screen->id ) {
                    $enqueue_form_scripts = true;
                }
            } else {
                // FRONTEND
                $enqueue_form_scripts = true;
            }

            if ( $enqueue_form_scripts ) {
                wp_enqueue_script( 'yith-wcbk-booking-form' );
                wp_enqueue_style( 'yith-wcbk-booking-form-style' );
            }
        }

    }
}
