<?php
/**
 * Product Helper class
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Product_Helper' ) ) {

    /**
     * YITH Booking for WooCommerce Premium Product Helper
     *
     * @since 1.0.0
     */
    class YITH_WCBK_Product_Helper {
        /**
         * Single instance of the class
         *
         * @var \YITH_WCBK_Product_Helper
         * @since 1.0.0
         */
        protected static $_instance;

        /**
         * Booking Product Type Name
         *
         * @type string
         */
        public static $prod_type = 'booking';

        public $product_meta_array = array();

        /**
         * Returns single instance of the class
         *
         * @return \YITH_WCBK_Product_Helper
         * @since 1.0.0
         */
        public static function get_instance() {
            if ( is_null( self::$_instance ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }

        /**
         * Constructor
         *
         */
        private function __construct() {
            $this->populate_meta_array();

            // Add Booking product to WC product type selector
            add_filter( 'product_type_selector', array( $this, 'product_type_selector' ) );

            // add tabs for product booking
            add_filter( 'woocommerce_product_data_tabs', array( $this, 'product_booking_tabs' ) );

            // Add options to general product data tab
            add_action( 'woocommerce_product_options_general_product_data', array( $this, 'add_options_to_general_product_data' ) );
            add_action( 'woocommerce_product_data_panels', array( $this, 'add_product_data_panels' ) );

            // save product meta
            add_action( 'woocommerce_process_product_meta_' . self::$prod_type, array( $this, 'save_product_meta' ) );

            // Remove Booking Services Metabox for products
            add_action( 'add_meta_boxes', array( $this, 'manage_meta_boxes' ) );
        }

        /**
         * Remove Booking Services Metabox for products
         *
         * @access public
         *
         * @param string $post_type Post type.
         *
         * @since  1.0.0
         */
        public function manage_meta_boxes( $post_type ) {
            remove_meta_box( YITH_WCBK_Post_Types::$service_tax . 'div', 'product', 'side' );
        }

        /**
         * Add options to general product data tab
         *
         * @access   public
         * @since    1.0.0
         * @author   Leanza Francesco <leanzafrancesco@gmail.com>
         * @author   Francesco Licandro
         *
         * @return void
         */
        public function add_options_to_general_product_data() {
            global $post;

            $post_type = get_post_type( $post->ID );
            if ( $post_type != 'product' )
                return;

            $args = array(
                'prod_type' => self::$prod_type,
                'post_id'   => $post->ID,
            );
            extract( $args );
            include( YITH_WCBK_TEMPLATE_PATH . 'admin/product-tabs/general-tab.php' );
        }

        /**
         * add data panels to products
         *
         * @return void
         */
        public function add_product_data_panels() {
            global $post;

            $tabs = array(
                'costs'     => 'yith_booking_costs_tab',
                'available' => 'yith_booking_available_tab',
                'persons'   => 'yith_booking_persons_tab',
                'services'  => 'yith_booking_services_tab',
            );

            $post_id = $post->ID;

            foreach ( $tabs as $key => $tab_id ) {
                //echo "<div id='{$tab_id}' class='panel woocommerce_options_panel show_if_{self::$prod_type}'>";
                echo "<div id='{$tab_id}' class='panel woocommerce_options_panel'>";
                include( YITH_WCBK_TEMPLATE_PATH . 'admin/product-tabs/' . $key . '-tab.php' );
                echo '</div>';
            }
        }

        public static function get_default_meta_array() {
            return array(
                // ------ Tab General --------------------------------------------------
                '_yith_booking_duration_type'                => '',
                '_yith_booking_duration'                     => 'int',
                '_yith_booking_duration_unit'                => '',
                '_yith_booking_minimum_duration'             => 'int',
                '_yith_booking_maximum_duration'             => 'zero_int',
                '_yith_booking_enable_calendar_range_picker' => 'checkbox',
                '_yith_booking_request_confirmation'         => 'checkbox',
                '_yith_booking_can_be_cancelled'             => 'checkbox',
                '_yith_booking_cancelled_duration'           => 'zero_int',
                '_yith_booking_cancelled_unit'               => '',
                '_yith_booking_location'                     => 'location',
                // ------ Tab Available --------------------------------------------------
                '_yith_booking_max_per_block'                => 'zero_int',
                '_yith_booking_allow_after'                  => 'zero_int',
                '_yith_booking_allow_after_unit'             => '',
                '_yith_booking_allow_until'                  => 'int',
                '_yith_booking_allow_until_unit'             => '',
                '_yith_booking_checkin'                      => '',
                '_yith_booking_checkout'                     => '',
                //'_yith_booking_allowed_start_days'         => '',
                '_yith_booking_default_start_date'           => '',
                '_yith_booking_default_start_date_custom'    => '',
                '_yith_booking_availability_range'           => 'availability_from_to',
                '_yith_booking_costs_range'                  => 'costs_from_to',
                // ------ Tab Costs --------------------------------------------------
                '_yith_booking_base_cost'                    => '',
                '_yith_booking_block_cost'                   => '',
                // ------ Tab Persons --------------------------------------------------
                '_yith_booking_has_persons'                  => 'checkbox',
                '_yith_booking_min_persons'                  => 'int',
                '_yith_booking_max_persons'                  => 'zero_int',
                '_yith_booking_multiply_costs_by_persons'    => 'checkbox',
                //'_yith_booking_count_persons_as_bookings'    => 'checkbox',
                '_yith_booking_enable_person_types'          => 'checkbox',
                '_yith_booking_person_types'                 => 'person_types',
                // ------ Tab Services --------------------------------------------------
                '_yith_booking_services'                     => 'services',
            );
        }

        /**
         * Populate the meta array
         *
         * @return void
         */
        public function populate_meta_array() {
            $this->product_meta_array = self::get_default_meta_array();
        }

        /**
         * Add tabs for booking products
         *
         * @param $tabs
         *
         * @return array
         * @author Francesco Licandro
         * @author Francesco Leanza
         */
        public function product_booking_tabs( $tabs ) {

            $new_tabs = array(
                'yith_booking_costs'     => array(
                    'label'  => __( 'Booking Costs', 'yith-booking-for-woocommerce' ),
                    'target' => 'yith_booking_costs_tab',
                    'class'  => array( 'show_if_' . self::$prod_type ),
                ),
                'yith_booking_available' => array(
                    'label'  => __( 'Booking Availability', 'yith-booking-for-woocommerce' ),
                    'target' => 'yith_booking_available_tab',
                    'class'  => array( 'show_if_' . self::$prod_type ),
                ),
                'yith_booking_persons'   => array(
                    'label'  => __( 'Booking People', 'yith-booking-for-woocommerce' ),
                    'target' => 'yith_booking_persons_tab',
                    'class'  => array( 'show_if_' . self::$prod_type ),
                ),
                'yith_booking_services'  => array(
                    'label'  => __( 'Booking Services', 'yith-booking-for-woocommerce' ),
                    'target' => 'yith_booking_services_tab',
                    'class'  => array( 'show_if_' . self::$prod_type ),
                ),
            );

            // add class to shipping tab
            if ( isset( $tabs[ 'shipping' ] ) && isset( $tabs[ 'shipping' ][ 'class' ] ) ) {
                if ( is_array( $tabs[ 'shipping' ][ 'class' ] ) ) {
                    $tabs[ 'shipping' ][ 'class' ][] = 'hide_if_' . self::$prod_type;
                }
            }

            $tabs = array_merge( $tabs, $new_tabs );

            return $tabs;
        }

        /**
         * add Booking Product type in product type selector [in product wc-metabox]
         *
         * @access   public
         * @since    1.0.0
         * @author   Leanza Francesco <leanzafrancesco@gmail.com>
         * @author   Francesco Licandro
         */
        public function product_type_selector( $types ) {
            $types[ self::$prod_type ] = _x( 'Booking Product', 'Admin: type of product', 'yith-booking-for-woocommerce' );

            return $types;
        }

        /**
         * Save Product Meta
         *
         * @param int $post_id
         *
         */
        public function save_product_meta( $post_id ) {
            foreach ( $this->product_meta_array as $key => $type ) {
                $value = '';
                switch ( $type ) {
                    case 'checkbox':
                        $value = isset( $_POST[ $key ] ) ? 'yes' : 'no';
                        break;
                    case 'int':
                        $value = ( isset( $_POST[ $key ] ) && $_POST[ $key ] >= 1 ) ? intval( $_POST[ $key ] ) : 1;
                        break;
                    case 'zero_int':
                        $value = ( isset( $_POST[ $key ] ) && $_POST[ $key ] >= 0 ) ? intval( $_POST[ $key ] ) : 0;
                        break;
                    case 'float':
                        $value = ( isset( $_POST[ $key ] ) && $_POST[ $key ] >= 0 ) ? floatval( $_POST[ $key ] ) : 0;
                        break;
                    case 'availability_from_to':
                        $ranges         = isset( $_POST[ $key ] ) ? $_POST[ $key ] : array();
                        $type_array     = isset( $ranges[ 'type' ] ) ? $ranges[ 'type' ] : array();
                        $from_array     = isset( $ranges[ 'from' ] ) ? $ranges[ 'from' ] : array();
                        $to_array       = isset( $ranges[ 'to' ] ) ? $ranges[ 'to' ] : array();
                        $bookable_array = isset( $ranges[ 'bookable' ] ) ? $ranges[ 'bookable' ] : array();
                        $days_enabled   = isset( $ranges[ 'days_enabled' ] ) ? $ranges[ 'days_enabled' ] : array();
                        $days           = isset( $ranges[ 'days' ] ) ? $ranges[ 'days' ] : array();

                        $availability_size = sizeof( $type_array );
                        $value             = array();
                        for ( $i = 1; $i < $availability_size; $i++ ) {
                            if ( !empty( $type_array[ $i ] ) && !empty( $from_array[ $i ] ) && !empty( $to_array[ $i ] ) && !empty( $bookable_array[ $i ] ) ) {
                                $current_range = array(
                                    'type'         => $type_array[ $i ],
                                    'from'         => $from_array[ $i ],
                                    'to'           => $to_array[ $i ],
                                    'bookable'     => $bookable_array[ $i ],
                                    'days_enabled' => $days_enabled[ $i ],
                                    'days'         => array(),
                                );

                                if ( $days_enabled[ $i ] === 'yes' ) {
                                    $current_range_days = array();
                                    for ( $j = 1; $j <= 7; $j++ ) {
                                        $current_range_days[ $j ] = isset( $days[ $j ][ $i ] ) ? $days[ $j ][ $i ] : $bookable_array[ $i ];
                                    }

                                    $current_range[ 'days' ] = $current_range_days;
                                }

                                $value[] = (object) $current_range;
                            }
                        }
                        break;
                    case 'costs_from_to':
                        $value               = isset( $_POST[ $key ] ) ? $_POST[ $key ] : array();
                        $type_array          = isset( $value[ 'type' ] ) ? $value[ 'type' ] : array();
                        $from_array          = isset( $value[ 'from' ] ) ? $value[ 'from' ] : array();
                        $to_array            = isset( $value[ 'to' ] ) ? $value[ 'to' ] : array();
                        $base_cost_operator  = isset( $value[ 'base_cost_operator' ] ) ? $value[ 'base_cost_operator' ] : array();
                        $base_cost           = isset( $value[ 'base_cost' ] ) ? $value[ 'base_cost' ] : array();
                        $block_cost_operator = isset( $value[ 'block_cost_operator' ] ) ? $value[ 'block_cost_operator' ] : array();
                        $block_cost          = isset( $value[ 'block_cost' ] ) ? $value[ 'block_cost' ] : array();

                        $costs_size = sizeof( $type_array );
                        $value      = array();
                        for ( $i = 1; $i < $costs_size; $i++ ) {
                            if ( isset( $type_array[ $i ] ) && isset( $from_array[ $i ] ) && isset( $to_array[ $i ] ) && isset( $base_cost_operator[ $i ] ) && isset( $block_cost_operator[ $i ] ) ) {
                                $single_base_cost  = !empty( $base_cost[ $i ] ) ? $base_cost[ $i ] : '';
                                $single_block_cost = !empty( $block_cost[ $i ] ) ? $block_cost[ $i ] : '';

                                $value[] = array(
                                    'type'                => $type_array[ $i ],
                                    'from'                => $from_array[ $i ],
                                    'to'                  => $to_array[ $i ],
                                    'base_cost_operator'  => $base_cost_operator[ $i ],
                                    'base_cost'           => $single_base_cost,
                                    'block_cost_operator' => $block_cost_operator[ $i ],
                                    'block_cost'          => $single_block_cost,
                                );
                            }
                        }
                        break;

                    case 'person_types':
                        $value = isset( $_POST[ $key ] ) ? $_POST[ $key ] : array();
                        break;
                    case 'array':
                        $value = isset( $_POST[ $key ] ) ? $_POST[ $key ] : array();
                        break;
                    case 'services':
                        $values = isset( $_POST[ $key ] ) ? $_POST[ $key ] : array();
                        $values = array_map( 'intval', $values );
                        $values = array_unique( $values );
                        wp_set_object_terms( $post_id, $values, YITH_WCBK_Post_Types::$service_tax );
                        continue;
                        break;
                        break;
                    default:
                        $value = ( isset( $_POST[ $key ] ) ) ? wc_clean( $_POST[ $key ] ) : '';
                        break;
                }
                if ( $type != 'services' )
                    update_post_meta( $post_id, $key, $value );

                // Save latitude e longitude of booking location
                if ( $type === 'location' && !!$value ) {
                    $location_info = YITH_WCBK()->maps->get_location_by_address( $value );
                    if ( isset( $location_info[ 'lat' ] ) && isset( $location_info[ 'lng' ] ) ) {
                        update_post_meta( $post_id, $key . '_lat', $location_info[ 'lat' ] );
                        update_post_meta( $post_id, $key . '_lng', $location_info[ 'lng' ] );
                    } else {
                        update_post_meta( $post_id, $key . '_lat', '' );
                        update_post_meta( $post_id, $key . '_lng', '' );
                    }
                }
            }
        }

        /**
         * Return true if the product is Booking Product
         *
         * @param bool|int|WP_Post|WC_Product $product
         *
         * @return bool
         */
        public static function is_booking( $product = false ) {
            $product_id = false;
            if ( false === $product ) {
                $product = $GLOBALS[ 'post' ];
                if ( !$product || !$product instanceof WP_Post )
                    return false;
                $product_id = $product->ID;
            } elseif ( is_numeric( $product ) ) {
                $product_id = absint( $product );
            } elseif ( $product instanceof WC_Product ) {
                $product_id = $product->get_id();
            } elseif ( !( $product instanceof WP_Post ) ) {
                $product_id = false;
            }

            if ( !$product_id )
                return false;

            $terms        = get_the_terms( $product_id, 'product_type' );
            $product_type = !empty( $terms ) && isset( current( $terms )->name ) ? sanitize_title( current( $terms )->name ) : 'simple';

            return self::$prod_type === $product_type;
        }
    }
}
