<?php

if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( !class_exists( 'YITH_WCBK_Email_Customer_Confirmed_Booking' ) ) :

    /**
     * Customer Email Confirmed Booking
     *
     * An email sent to the customer when a new booking is confirmed
     *
     * @class       YITH_WCBK_Email_Customer_Confirmed_Booking
     * @version     1.0.0
     */
    class YITH_WCBK_Email_Customer_Confirmed_Booking extends WC_Email {

        /**
         * @var YITH_WCBK_Booking
         */
        public $object;

        /**
         * Constructor.
         */
        public function __construct() {
            $this->id             = 'yith_wcbk_customer_confirmed_booking';
            $this->title          = __( 'Confirmed Booking', 'yith-booking-for-woocommerce' );
            $this->description    = __( 'This email is sent to customers when a booking is confirmed.', 'yith-booking-for-woocommerce' );
            $this->heading        = __( 'Confirmed Booking', 'yith-booking-for-woocommerce' );
            $this->subject        = __( 'Booking #{booking_id} is now confirmed', 'yith-booking-for-woocommerce' );
            $this->customer_email = true;

            $this->template_base  = YITH_WCBK_TEMPLATE_PATH;
            $this->template_html  = 'emails/customer-confirmed-booking.php';
            $this->template_plain = 'emails/plain/customer-confirmed-booking.php';

            // Triggers for this email
            add_action( 'yith_wcbk_booking_status_confirmed_notification', array( $this, 'trigger' ) );

            // Call parent constructor
            parent::__construct();
        }

        /**
         * Trigger.
         *
         * @param int $booking_id
         */
        public function trigger( $booking_id ) {
            if ( $booking_id ) {
                $this->object = yith_get_booking( $booking_id );
                if ( $this->object->is_valid() ) {
                    $this->find[ 'booking-id' ]        = '{booking_id}';
                    $this->find[ 'booking-status' ]    = '{status}';
                    $this->replace[ 'booking-id' ]     = absint( $booking_id );
                    $this->replace[ 'booking-status' ] = $this->object->get_status_text();


                    if ( $this->object->order_id && $order = wc_get_order( $this->object->order_id ) ) {
                        $this->recipient = yit_get_prop( $order, 'billing_email', true );
                    } elseif ( $this->object->user_id && $user_data = get_userdata( $this->object->user_id ) ) {
                        $this->recipient = $user_data->user_email;
                    }

                    if ( !$this->is_enabled() || !$this->get_recipient() ) {
                        return;
                    }
                    $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
                }
            }
        }

        /**
         * Get content html.
         *
         * @access public
         * @return string
         */
        public function get_content_html() {
            return wc_get_template_html( $this->template_html, array(
                'booking'       => $this->object,
                'email_heading' => $this->get_heading(),
                'sent_to_admin' => !$this->is_customer_email(),
                'plain_text'    => false,
                'email'         => $this
            ), '', $this->template_base );
        }

        /**
         * Get content plain.
         *
         * @access public
         * @return string
         */
        public function get_content_plain() {
            return wc_get_template_html( $this->template_plain, array(
                'booking'       => $this->object,
                'email_heading' => $this->get_heading(),
                'sent_to_admin' => !$this->is_customer_email(),
                'plain_text'    => true,
                'email'         => $this
            ), '', $this->template_base );
        }

        /**
         * Initialise settings form fields.
         */
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled'    => array(
                    'title'   => __( 'Enable/Disable', 'yith-booking-for-woocommerce' ),
                    'type'    => 'checkbox',
                    'label'   => __( 'Enable this email notification', 'yith-booking-for-woocommerce' ),
                    'default' => 'yes'
                ),
                'subject'    => array(
                    'title'       => __( 'Subject', 'yith-booking-for-woocommerce' ),
                    'type'        => 'text',
                    'description' => sprintf( __( 'This controls the email subject line. Leave blank to use the default subject: <code>%s</code>.', 'woocommerce' ), $this->subject ),
                    'placeholder' => '',
                    'default'     => '',
                    'desc_tip'    => true
                ),
                'heading'    => array(
                    'title'       => __( 'Email Heading', 'yith-booking-for-woocommerce' ),
                    'type'        => 'text',
                    'description' => sprintf( __( 'This controls the main heading in the email notification. Leave blank to use the default heading: <code>%s</code>.', 'woocommerce' ), $this->heading ),
                    'placeholder' => '',
                    'default'     => '',
                    'desc_tip'    => true
                ),
                'email_type' => array(
                    'title'       => __( 'Email type', 'yith-booking-for-woocommerce' ),
                    'type'        => 'select',
                    'description' => __( 'Choose which email format to send.', 'yith-booking-for-woocommerce' ),
                    'default'     => 'html',
                    'class'       => 'email_type wc-enhanced-select',
                    'options'     => $this->get_email_type_options(),
                    'desc_tip'    => true
                )
            );
        }
    }

endif;

return new YITH_WCBK_Email_Customer_Confirmed_Booking();
